import { EnhancedState, normalizeArray, copy, CollectionPersistent, isValidObject, ComputedTracker, removeProperties, LogCodeManager, GroupObserver, StateObserver, defineConfig, } from '../../internal';
export class Group extends EnhancedState {
    constructor(collection, initialItems = [], config = {}) {
        super(collection.agileInstance(), initialItems, config);
        this._output = [];
        this.observers = {};
        this.notFoundItemKeys = [];
        this.observers['value'] = new StateObserver(this, {
            key: config.key,
        });
        this.observers['output'] = new GroupObserver(this, {
            key: config.key,
        });
        this.collection = () => collection;
        this.addSideEffect(Group.rebuildGroupSideEffectKey, () => this.rebuild());
        this.rebuild();
    }
    get output() {
        ComputedTracker.tracked(this.observers['output']);
        return copy(this._output);
    }
    set output(value) {
        LogCodeManager.log('1C:03:00', [this._key]);
    }
    has(itemKey) {
        return this.value.findIndex((key) => key === itemKey) !== -1;
    }
    get size() {
        return this.value.length;
    }
    remove(itemKeys, config = {}) {
        const _itemKeys = normalizeArray(itemKeys);
        const notExistingItemKeysInCollection = [];
        const notExistingItemKeys = [];
        let newGroupValue = copy(this.nextStateValue);
        _itemKeys.forEach((itemKey) => {
            if (!newGroupValue.includes(itemKey)) {
                notExistingItemKeys.push(itemKey);
                notExistingItemKeysInCollection.push(itemKey);
                return;
            }
            if (!this.collection().getItem(itemKey))
                notExistingItemKeysInCollection.push(itemKey);
            newGroupValue = newGroupValue.filter((key) => key !== itemKey);
        });
        if (notExistingItemKeys.length >= _itemKeys.length)
            return this;
        if (notExistingItemKeysInCollection.length >= _itemKeys.length)
            config.background = true;
        this.set(newGroupValue, config);
        return this;
    }
    add(itemKeys, config = {}) {
        const _itemKeys = normalizeArray(itemKeys);
        const notExistingItemKeysInCollection = [];
        const existingItemKeys = [];
        let newGroupValue = copy(this.nextStateValue);
        defineConfig(config, {
            method: 'push',
            overwrite: false,
        });
        _itemKeys.forEach((itemKey) => {
            if (!this.collection().getItem(itemKey))
                notExistingItemKeysInCollection.push(itemKey);
            if (newGroupValue.includes(itemKey)) {
                if (config.overwrite) {
                    newGroupValue = newGroupValue.filter((key) => key !== itemKey);
                }
                else {
                    existingItemKeys.push(itemKey);
                    return;
                }
            }
            newGroupValue[config.method || 'push'](itemKey);
        });
        if (existingItemKeys.length >= _itemKeys.length)
            return this;
        if (notExistingItemKeysInCollection.concat(existingItemKeys).length >=
            _itemKeys.length)
            config.background = true;
        this.set(newGroupValue, removeProperties(config, ['method', 'overwrite']));
        return this;
    }
    replace(oldItemKey, newItemKey, config = {}) {
        const newGroupValue = copy(this._value);
        newGroupValue.splice(newGroupValue.indexOf(oldItemKey), 1, newItemKey);
        this.set(newGroupValue, config);
        return this;
    }
    getItems() {
        return this.value
            .map((itemKey) => this.collection().getItem(itemKey))
            .filter((item) => item !== undefined);
    }
    persist(keyOrConfig = {}, config = {}) {
        let _config;
        let key;
        if (isValidObject(keyOrConfig)) {
            _config = keyOrConfig;
            key = this._key;
        }
        else {
            _config = config || {};
            key = keyOrConfig;
        }
        _config = defineConfig(_config, {
            loadValue: true,
            followCollectionPersistKeyPattern: true,
            storageKeys: [],
            defaultStorageKey: null,
        });
        if (_config.followCollectionPersistKeyPattern) {
            key = CollectionPersistent.getGroupStorageKey(key || this._key, this.collection()._key);
        }
        super.persist(key, {
            loadValue: _config.loadValue,
            storageKeys: _config.storageKeys,
            defaultStorageKey: _config.defaultStorageKey,
        });
        return this;
    }
    rebuild(config = {}) {
        const notFoundItemKeys = [];
        const groupItems = [];
        if (!this.collection().isInstantiated)
            return this;
        this._value.forEach((itemKey) => {
            const item = this.collection().getItem(itemKey);
            if (item != null)
                groupItems.push(item);
            else
                notFoundItemKeys.push(itemKey);
        });
        if (notFoundItemKeys.length > 0) {
            LogCodeManager.log('1C:02:00', [this.collection()._key, this._key], notFoundItemKeys);
        }
        this.notFoundItemKeys = notFoundItemKeys;
        this.observers['output'].ingestItems(groupItems, config);
        return this;
    }
}
Group.rebuildGroupSideEffectKey = 'rebuildGroup';
